@@ -305,6 +305,10 @@ int SSL_callback_SSL_verify(int ok, X509_STORE_CTX *ctx)
     int verify   = con->ctx->verify_mode;
     int depth    = con->ctx->verify_depth;
 
+#if defined(SSL_OP_NO_TLSv1_3)
+    con->pha_state = PHA_COMPLETE;
+#endif
+
     if (verify == SSL_CVERIFY_UNSET ||
         verify == SSL_CVERIFY_NONE)
         return 1;
@@ -386,12 +390,24 @@ int SSL_callback_SSL_verify(int ok, X509_STORE_CTX *ctx)
 void SSL_callback_handshake(const SSL *ssl, int where, int rc)
 {
     tcn_ssl_conn_t *con = (tcn_ssl_conn_t *)SSL_get_app_data(ssl);
+#ifdef HAVE_TLSV1_3
+    const SSL_SESSION *session = SSL_get_session(ssl);
+#endif
 
     /* Retrieve the conn_rec and the associated SSLConnRec. */
     if (con == NULL) {
         return;
     }
 
+#ifdef HAVE_TLSV1_3
+    /* TLS 1.3 does not use renegotiation so do not update the renegotiation
+     * state once we know we are using TLS 1.3. */
+    if (session != NULL) {
+        if (SSL_SESSION_get_protocol_version(session) == TLS1_3_VERSION) {
+            return;
+        }
+    }
+#endif
 
     /* If the reneg state is to reject renegotiations, check the SSL
      * state machine and move to ABORT if a Client Hello is being
@@ -405,7 +421,6 @@ void SSL_callback_handshake(const SSL *ssl, int where, int rc)
     else if ((where & SSL_CB_HANDSHAKE_DONE) && con->reneg_state == RENEG_INIT) {
         con->reneg_state = RENEG_REJECT;
     }
-
 }
 
 int SSL_callback_next_protos(SSL *ssl, const unsigned char **data,
@@ -595,7 +610,7 @@ static int parse_asn1_length(unsigned char **asn1, int *len) {
         // Single byte length
         *len = **asn1;
     }
-    
+
     (*asn1)++;
 
     return 0;
